#!/usr/bin/env bash

script_name="$(basename $(realpath $0))"
working_dir="$(dirname $(realpath $0))"

conf_file="${working_dir}/conf/${script_name%%.*}.conf"

LOCK_DIR="/tmp/$(basename $0).$$.LOCK"
log_file="${LOCK_DIR}/${script_name%%.*}.log"

backup_date="$(date +%Y-%m-%d-%H%M)"
declare -a files_backuped_arr=()
declare -a dirs_backuped_arr=()

# ----------
# Base Function(s)
# ----------

usage() {


   [[ -n  "$1" ]] && error "$1"


   [[ $terminal ]] && echo -e "
\033[1mUsage:\033[m

   $(basename $0)

\033[1mDescription\033[m

   Scripts syncs nextcloud accounts. 

\033[1mOptions\033[m

   All Options and Parameters read from configuration file.

\033[1mFiles\033[m

   Configuration file:
      $conf_file


"

   clean_up 1

}

clean_up() {

   # Perform program exit housekeeping
   rm -rf "$LOCK_DIR"
   blank_line
   exit $1
}


echononl(){
   if $terminal ; then
      echo X\\c > /tmp/shprompt$$
      if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
         echo -e -n "   $*\\c" 1>&2
      else
         echo -e -n "   $*" 1>&2
      fi
      rm /tmp/shprompt$$
   fi
}


fatal(){
   echo ""
   if $terminal ; then
      echo -e "   [ \033[31m\033[1mFatal\033[m ] $*"
   else
      echo -e "   [ Fatal ] $*"
   fi
   echo ""
   if $terminal ; then
      echo -e "             \033[1mScript terminated\033[m.."
   else
      echo -e "             Script terminated.."
   fi
   echo ""
   rm -rf $LOCK_DIR
   exit 1
}

error (){
   echo ""
   if $terminal ; then
      echo -e "   [ \033[31m\033[1mError\033[m ] $*"
   else
      echo "   [ Error ] $*"
   fi
   echo ""
}

warn (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[33m\033[1mWarn\033[m ]  $*"
      else
         echo "   [ Warn ]  $*"
      fi
      echo ""
   fi
}

info (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[32m\033[1mInfo\033[m ]  $*"
      else
         echo "   [ Info ]  $*"
      fi
      echo ""
   fi
}

ok (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[32m\033[1mOk\033[m ]    $*"
      else
         echo "   [ Ok ]    $*"
      fi
      echo ""
   fi
}

echo_done() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mdone\033[m ]"
   fi
}
echo_ok() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mok\033[m ]"
   fi
}
echo_warn() {
   if $terminal ; then
      echo -e "\033[75G[ \033[33mwarn\033[m ]"
   fi
}
echo_failed(){
   if $terminal ; then
      echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
   fi
}
echo_skipped() {
   if $terminal ; then
      echo -e "\033[75G[ \033[90m\033[1mskipped\033[m ]"
   fi
}
echo_wait(){
   if $terminal ; then
      echo -en "\033[75G[ \033[5m\033[1m...\033[m ]"
   fi
}

trim() {
    local var="$*"
    var="${var#"${var%%[![:space:]]*}"}"   # remove leading whitespace characters
    var="${var%"${var##*[![:space:]]}"}"   # remove trailing whitespace characters
    echo -n "$var"
}

blank_line() {
   if $terminal ; then
      echo ""
   fi
}

# - Backup file or directory
# -
backup() {

   is_directory=false

   if [[ -z "$1" ]] ; then
      error "No file/directory for backup given!"
      return
   elif [[ -h "$1" ]] ; then
      _source="$(realpath $1)"
      warn "'$1' is a symlink to $_source."
   elif [[ ! -d "$1" ]] && [[ ! -f "$1" ]] ; then
      warn "'$1' not found. Backup skipped."
      return
   else
      _source="$(realpath $1)"
   fi

   if [[ -d "$_source" ]]; then
      is_directory=true
      echononl "Backup directory '$_source' .."
   else
      echononl "Backup file '$_source' .."
   fi

   cp -a "$_source" "${_source}.$backup_date" > $log_file 2>&1
   if [[ $? -eq 0 ]] ; then
      echo_ok
      if $is_directory ; then
         dirs_backuped_arr+=("$_source")
      else
         files_backuped_arr+=("$_source")
      fi
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
}


# - Remove script generated backups, if source wasn't changed
# -
rm_unchanged_backup() {

   if [[ ${#files_backuped_arr[@]} -gt 0 ]] ; then
      for _file in "${files_backuped_arr[@]}" ; do
         if  $(diff "$_file" "${_file}.$backup_date" > /dev/null 2>&1) ; then
            echononl "File '$(basename "${_file}")' wasn't changed.\n   Delete the previous generated backup. .."
            rm "${_file}.$backup_date" > "$log_file" 2>&1
            if [[ $? -eq 0 ]]; then
               echo_ok
            else
               echo_failed
               error "$(cat "$log_file")"
            fi
            blank_line
         fi
      done
   fi
   if [[ ${#dirs_backuped_arr[@]} -gt 0 ]] ; then
      for _dir in "${dirs_backuped_arr[@]}" ; do
         if $(diff -Nur "$_dir" "${_dir}.$backup_date" > /dev/null 2>&1) ; then
            echononl "Directory '$(basename "${_dir}")' wasn't changed.\n   Delete the previous generated backup. .."
            rm -rf "${_dir}.$backup_date" > "$log_file" 2>&1
            if [[ $? -eq 0 ]]; then
               echo_ok
            else
               echo_failed
               error "$(cat "$log_file")"
            fi
            blank_line
         fi
      done
   fi
}



# ----------
# - Jobhandling 
# ----------

# - If job already runs, stop execution..
# -
if mkdir "$LOCK_DIR" 2> /dev/null ; then

   # - Remove lockdir when the script finishes, or when it receives a signal
   # -
   trap clean_up SIGHUP SIGINT SIGTERM

else

   msg="A previos instance of script \"`basename $0`\" seems already be running."

   echo ""
   if $terminal ; then
      echo -e "[ \033[31m\033[1mFatal\033[m ]: $msg"
      echo ""
      echo -e "           \033[31m\033[1mScript was interupted\033[m!"
   else
      echo "   [ Fatal ]: $msg"
      echo ""
      echo "              Script was interupted!"
   fi
   echo

   exit 1

fi


# ----------
# - Some checks ..
# ----------

# - Running in a terminal?
# -
if [[ -t 1 ]] ; then
   terminal=true
else
   terminal=false
fi

# -Is systemd supported on this system?
# -
systemd_supported=false
systemd=$(which systemd)
systemctl=$(which systemctl)

if [[ -n "$systemd" ]] && [[ -n "$systemctl" ]] ; then
   systemd_supported=true
fi

# - Print help?
# -
if [[ "$(trim $*)" = "-h" ]] || [[ "$(trim $*)" = "--help" ]] ; then
    usage
fi

if [[ -z "$(which basename)" ]]; then
   fatal 'It seems "basename" is not installed, but needed!'
fi

if [[ -z "$(which realpath)" ]]; then
   fatal 'It seems "realpath" is not installed, but needed!'
fi


# ==========
# - Begin Main Script
# ==========

# ----------
# - Headline
# ----------

if $terminal ; then
   echo ""
   echo -e "\033[1m----------\033[m"
   echo -e "\033[32m\033[1mRunning script \033[m\033[1m$script_name\033[32m .. \033[m"
   echo -e "\033[1m----------\033[m"
fi


# ----------
#  Read Configurations from $conf_file
# ----------


# - Give your default values here
# -
DEFAULT_PARAMS="--non-interactive"
CMD="$(which nextcloudcmd)"

if [[ ! -x "$CMD" ]] ; then
   fatal "Command 'nextcloudcmd' not found"
fi

if [[ -f "$conf_file" ]]; then
   source "$conf_file"
else
   fatal "No configuration file '$conf_file' present."
fi

[[ -z "$(trim "$NEXTCLOUD_SERVER_URL")" ]] && fatal "Parameter 'NEXTCLOUD_SERVER_URL' not set but is reqired!"
[[ -z "$(trim "$NEXTCLOUD_ACCOUNTS")" ]] && fatal "No accounts for syncing given - Parmeter 'NEXTCLOUD_ACCOUNTS'!"
[[ -z "$(trim "$LOCAL_BASE_SYNC_DIR")" ]] && fatal "No local sync directory given - Parameter 'LOCAL_BASE_SYNC_DIR'!"
[[ -z "$(trim "$LOCAL_BASE_SYNC_DIR")" ]] && PARAMS="${DEFAULT_PARAMS}"

declare -a nextcloud_account_arr
for _account in $NEXTCLOUD_ACCOUNTS ; do
   nextcloud_account_arr+=("$_account")
done




# ----------
# - Some pre-script tasks ..
# ----------

if $terminal ; then
   echo ""
   echo ""
   echo -e "   \033[1mDoing some pre-script tasks ..\033[m"
   echo ""
fi

echononl "Create local sync directory '$LOCAL_BASE_SYNC_DIR'.."
if [[ ! -d "$LOCAL_BASE_SYNC_DIR" ]] ; then
   mkdir "$LOCAL_BASE_SYNC_DIR" > $log_file 2>&1
   if [[ $? -eq 0 ]] ; then
       echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi


# ----------
# - Main part of script
# ----------

if $terminal ; then
   echo ""
   echo ""
   echo -e "   \033[1mMain part of script ..\033[m"
   echo ""
fi

for _val in "${nextcloud_account_arr[@]}" ; do
   IFS=':' read -a _val_arr <<< "${_val}"

   LOCAL_SYNC_DIR="${LOCAL_BASE_SYNC_DIR}/${_val_arr[0]}"
   if [[ ! -d "$LOCAL_SYNC_DIR" ]] ; then
      echononl "Create directory '$LOCAL_SYNC_DIR'.."
      mkdir "$LOCAL_SYNC_DIR" > $log_file 2>&1
      if [[ $? -eq 0 ]] ; then
         echo_ok
      else
         echo_failed
         error "$(cat "$log_file")"
         continue
      fi
   fi

   echononl "Backup nextcloud account '${_val_arr[0]}'.."
   $CMD $PARAMS -u "${_val_arr[0]}" -p "${_val_arr[1]}" $LOCAL_SYNC_DIR $NEXTCLOUD_SERVER_URL > $log_file 2>&1
   if [[ $? -eq 0 ]] ; then
       echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
done





# ----------
# - Some post-script tasks ..
# ----------

if $terminal ; then
   echo ""
   echo ""
   echo -e "   \033[1mDoing some post-script tasks ..\033[m"
   echo ""
fi


# - Remove generated backup file if source wasn't changed.
# -
rm_unchanged_backup

clean_up 0
